/****************************************************************************
** Meta object code from reading C++ file 'process_table_view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../gui/process_table_view.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'process_table_view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16ProcessTableViewE_t {};
} // unnamed namespace

template <> constexpr inline auto ProcessTableView::qt_create_metaobjectdata<qt_meta_tag_ZN16ProcessTableViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ProcessTableView",
        "signalModelUpdated",
        "",
        "signalHeadchanged",
        "endProcess",
        "pauseProcess",
        "resumeProcess",
        "openExecDirWithFM",
        "showProperties",
        "killProcess",
        "search",
        "text",
        "switchDisplayMode",
        "FilterType",
        "type",
        "changeProcessPriority",
        "priority",
        "onThemeTypeChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalModelUpdated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signalHeadchanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'endProcess'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'pauseProcess'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resumeProcess'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openExecDirWithFM'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showProperties'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'killProcess'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'search'
        QtMocHelpers::SlotData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'switchDisplayMode'
        QtMocHelpers::SlotData<void(FilterType)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'changeProcessPriority'
        QtMocHelpers::SlotData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'onThemeTypeChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProcessTableView, qt_meta_tag_ZN16ProcessTableViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ProcessTableView::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseTableView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProcessTableViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProcessTableViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16ProcessTableViewE_t>.metaTypes,
    nullptr
} };

void ProcessTableView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProcessTableView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalModelUpdated(); break;
        case 1: _t->signalHeadchanged(); break;
        case 2: _t->endProcess(); break;
        case 3: _t->pauseProcess(); break;
        case 4: _t->resumeProcess(); break;
        case 5: _t->openExecDirWithFM(); break;
        case 6: _t->showProperties(); break;
        case 7: _t->killProcess(); break;
        case 8: _t->search((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->switchDisplayMode((*reinterpret_cast<std::add_pointer_t<FilterType>>(_a[1]))); break;
        case 10: _t->changeProcessPriority((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->onThemeTypeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProcessTableView::*)()>(_a, &ProcessTableView::signalModelUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessTableView::*)()>(_a, &ProcessTableView::signalHeadchanged, 1))
            return;
    }
}

const QMetaObject *ProcessTableView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ProcessTableView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16ProcessTableViewE_t>.strings))
        return static_cast<void*>(this);
    return BaseTableView::qt_metacast(_clname);
}

int ProcessTableView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseTableView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void ProcessTableView::signalModelUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ProcessTableView::signalHeadchanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
