/****************************************************************************
** Meta object code from reading C++ file 'process_db.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../process/process_db.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'process_db.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4core7process9ProcessDBE_t {};
} // unnamed namespace

template <> constexpr inline auto core::process::ProcessDB::qt_create_metaobjectdata<qt_meta_tag_ZN4core7process9ProcessDBE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "core::process::ProcessDB",
        "processListUpdated",
        "",
        "processEnded",
        "pid_t",
        "pid",
        "processPaused",
        "state",
        "processResumed",
        "processKilled",
        "processPriorityChanged",
        "priority",
        "priorityPromoteResultReady",
        "ErrorContext",
        "ec",
        "processControlResultReady",
        "filterTypeChanged",
        "FilterType",
        "filter",
        "signalProcessPrioritysetChanged",
        "endProcess",
        "pauseProcess",
        "resumeProcess",
        "killProcess",
        "setProcessPriority"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'processListUpdated'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'processEnded'
        QtMocHelpers::SignalData<void(pid_t)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'processPaused'
        QtMocHelpers::SignalData<void(pid_t, char)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Char, 7 },
        }}),
        // Signal 'processResumed'
        QtMocHelpers::SignalData<void(pid_t, char)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Char, 7 },
        }}),
        // Signal 'processKilled'
        QtMocHelpers::SignalData<void(pid_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'processPriorityChanged'
        QtMocHelpers::SignalData<void(pid_t, int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 11 },
        }}),
        // Signal 'priorityPromoteResultReady'
        QtMocHelpers::SignalData<void(const ErrorContext &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'processControlResultReady'
        QtMocHelpers::SignalData<void(const ErrorContext &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'filterTypeChanged'
        QtMocHelpers::SignalData<void(FilterType)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'signalProcessPrioritysetChanged'
        QtMocHelpers::SignalData<void(pid_t, int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'endProcess'
        QtMocHelpers::SlotData<void(pid_t)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'pauseProcess'
        QtMocHelpers::SlotData<void(pid_t)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'resumeProcess'
        QtMocHelpers::SlotData<void(pid_t)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'killProcess'
        QtMocHelpers::SlotData<void(pid_t)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Slot 'setProcessPriority'
        QtMocHelpers::SlotData<void(pid_t, int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { QMetaType::Int, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ProcessDB, qt_meta_tag_ZN4core7process9ProcessDBE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject core::process::ProcessDB::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4core7process9ProcessDBE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4core7process9ProcessDBE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4core7process9ProcessDBE_t>.metaTypes,
    nullptr
} };

void core::process::ProcessDB::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ProcessDB *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->processListUpdated(); break;
        case 1: _t->processEnded((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 2: _t->processPaused((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<char>>(_a[2]))); break;
        case 3: _t->processResumed((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<char>>(_a[2]))); break;
        case 4: _t->processKilled((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 5: _t->processPriorityChanged((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->priorityPromoteResultReady((*reinterpret_cast<std::add_pointer_t<ErrorContext>>(_a[1]))); break;
        case 7: _t->processControlResultReady((*reinterpret_cast<std::add_pointer_t<ErrorContext>>(_a[1]))); break;
        case 8: _t->filterTypeChanged((*reinterpret_cast<std::add_pointer_t<FilterType>>(_a[1]))); break;
        case 9: _t->signalProcessPrioritysetChanged((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->endProcess((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 11: _t->pauseProcess((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 12: _t->resumeProcess((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 13: _t->killProcess((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1]))); break;
        case 14: _t->setProcessPriority((*reinterpret_cast<std::add_pointer_t<pid_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ErrorContext >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< ErrorContext >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)()>(_a, &ProcessDB::processListUpdated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t )>(_a, &ProcessDB::processEnded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t , char )>(_a, &ProcessDB::processPaused, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t , char )>(_a, &ProcessDB::processResumed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t )>(_a, &ProcessDB::processKilled, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t , int )>(_a, &ProcessDB::processPriorityChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(const ErrorContext & )>(_a, &ProcessDB::priorityPromoteResultReady, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(const ErrorContext & )>(_a, &ProcessDB::processControlResultReady, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(FilterType )>(_a, &ProcessDB::filterTypeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ProcessDB::*)(pid_t , int )>(_a, &ProcessDB::signalProcessPrioritysetChanged, 9))
            return;
    }
}

const QMetaObject *core::process::ProcessDB::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *core::process::ProcessDB::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4core7process9ProcessDBE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int core::process::ProcessDB::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void core::process::ProcessDB::processListUpdated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void core::process::ProcessDB::processEnded(pid_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void core::process::ProcessDB::processPaused(pid_t _t1, char _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void core::process::ProcessDB::processResumed(pid_t _t1, char _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void core::process::ProcessDB::processKilled(pid_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void core::process::ProcessDB::processPriorityChanged(pid_t _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void core::process::ProcessDB::priorityPromoteResultReady(const ErrorContext & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void core::process::ProcessDB::processControlResultReady(const ErrorContext & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void core::process::ProcessDB::filterTypeChanged(FilterType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void core::process::ProcessDB::signalProcessPrioritysetChanged(pid_t _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
