/****************************************************************************
** Meta object code from reading C++ file 'systemd1_manager_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../dbus/systemd1_manager_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'systemd1_manager_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Systemd1ManagerInterface::qt_create_metaobjectdata<qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Systemd1ManagerInterface",
        "ListUnitFiles",
        "std::pair<ErrorContext,UnitFileInfoList>",
        "",
        "ListUnits",
        "std::pair<ErrorContext,UnitInfoList>",
        "GetUnit",
        "std::pair<ErrorContext,QDBusObjectPath>",
        "path",
        "StartUnit",
        "name",
        "mode",
        "StopUnit",
        "RestartUnit",
        "GetUnitFileState",
        "std::pair<ErrorContext,QString>",
        "unit",
        "EnableUnitFiles",
        "std::pair<ErrorContext,EnableUnitFilesResult>",
        "nlist",
        "runtime",
        "force",
        "DisableUnitFiles",
        "std::pair<ErrorContext,DisableUnitFilesResult>"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ListUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,UnitFileInfoList>()>(1, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'ListUnits'
        QtMocHelpers::SlotData<QPair<ErrorContext,UnitInfoList>()>(4, 3, QMC::AccessPublic, 0x80000000 | 5),
        // Slot 'GetUnit'
        QtMocHelpers::SlotData<QPair<ErrorContext,QDBusObjectPath>(const QString &)>(6, 3, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'StartUnit'
        QtMocHelpers::SlotData<QPair<ErrorContext,QDBusObjectPath>(const QString &, const QString &)>(9, 3, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'StopUnit'
        QtMocHelpers::SlotData<QPair<ErrorContext,QDBusObjectPath>(const QString &, const QString &)>(12, 3, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'RestartUnit'
        QtMocHelpers::SlotData<QPair<ErrorContext,QDBusObjectPath>(const QString &, const QString &)>(13, 3, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'GetUnitFileState'
        QtMocHelpers::SlotData<QPair<ErrorContext,QString>(const QString &)>(14, 3, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 16 },
        }}),
        // Slot 'EnableUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,EnableUnitFilesResult>(const QStringList &, bool, bool)>(17, 3, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::QStringList, 19 }, { QMetaType::Bool, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'EnableUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,EnableUnitFilesResult>(const QStringList &, bool)>(17, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 18, {{
            { QMetaType::QStringList, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'EnableUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,EnableUnitFilesResult>(const QStringList &)>(17, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 18, {{
            { QMetaType::QStringList, 19 },
        }}),
        // Slot 'DisableUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,DisableUnitFilesResult>(const QStringList &, bool)>(22, 3, QMC::AccessPublic, 0x80000000 | 23, {{
            { QMetaType::QStringList, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'DisableUnitFiles'
        QtMocHelpers::SlotData<QPair<ErrorContext,DisableUnitFilesResult>(const QStringList &)>(22, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 23, {{
            { QMetaType::QStringList, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Systemd1ManagerInterface, qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Systemd1ManagerInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>.metaTypes,
    nullptr
} };

void Systemd1ManagerInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Systemd1ManagerInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { std::pair<ErrorContext,UnitFileInfoList> _r = _t->ListUnitFiles();
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,UnitFileInfoList>*>(_a[0]) = std::move(_r); }  break;
        case 1: { std::pair<ErrorContext,UnitInfoList> _r = _t->ListUnits();
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,UnitInfoList>*>(_a[0]) = std::move(_r); }  break;
        case 2: { std::pair<ErrorContext,QDBusObjectPath> _r = _t->GetUnit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 3: { std::pair<ErrorContext,QDBusObjectPath> _r = _t->StartUnit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 4: { std::pair<ErrorContext,QDBusObjectPath> _r = _t->StopUnit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 5: { std::pair<ErrorContext,QDBusObjectPath> _r = _t->RestartUnit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,QDBusObjectPath>*>(_a[0]) = std::move(_r); }  break;
        case 6: { std::pair<ErrorContext,QString> _r = _t->GetUnitFileState((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: { std::pair<ErrorContext,EnableUnitFilesResult> _r = _t->EnableUnitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,EnableUnitFilesResult>*>(_a[0]) = std::move(_r); }  break;
        case 8: { std::pair<ErrorContext,EnableUnitFilesResult> _r = _t->EnableUnitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,EnableUnitFilesResult>*>(_a[0]) = std::move(_r); }  break;
        case 9: { std::pair<ErrorContext,EnableUnitFilesResult> _r = _t->EnableUnitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,EnableUnitFilesResult>*>(_a[0]) = std::move(_r); }  break;
        case 10: { std::pair<ErrorContext,DisableUnitFilesResult> _r = _t->DisableUnitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,DisableUnitFilesResult>*>(_a[0]) = std::move(_r); }  break;
        case 11: { std::pair<ErrorContext,DisableUnitFilesResult> _r = _t->DisableUnitFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::pair<ErrorContext,DisableUnitFilesResult>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *Systemd1ManagerInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Systemd1ManagerInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24Systemd1ManagerInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int Systemd1ManagerInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
